@@ -21,12 +21,15 @@ from sqlalchemy.testing.exclusions import \
      LambdaPredicate,\
      requires_tag
 
+
 def no_support(db, reason):
     return SpecPredicate(db, description=reason)
 
+
 def exclude(db, op, spec, description=None):
     return SpecPredicate(db, op, spec, description=description)
 
+
 class DefaultRequirements(SuiteRequirements):
     @property
     def deferrable_or_no_constraints(self):
@@ -44,6 +47,21 @@ class DefaultRequirements(SuiteRequirements):
 
         return exclusions.open()
 
+    @property
+    def enforces_check_constraints(self):
+        """Target database must also enforce check constraints."""
+
+        def mysql_not_mariadb_102(config):
+            return against(config, "mysql") and (
+                not config.db.dialect._is_mariadb or
+                config.db.dialect.server_version_info < (5, 5, 5, 10, 2)
+            )
+
+        return self.check_constraints + fails_on(
+            mysql_not_mariadb_102,
+            "check constraints don't enforce on MySQL, MariaDB<10.2"
+        )
+
     @property
     def named_constraints(self):
         """target database must support names for constraints."""
@@ -113,21 +131,20 @@ class DefaultRequirements(SuiteRequirements):
         without being in the context of a typed column.
 
         """
-        return skip_if(["firebird", "mssql+mxodbc"],
-                "not supported by driver")
+        return skip_if(["firebird", "mssql+mxodbc"], "not supported by driver")
 
     @property
     def identity(self):
         """Target database must support GENERATED AS IDENTITY or a facsimile.
 
         Includes GENERATED AS IDENTITY, AUTOINCREMENT, AUTO_INCREMENT, or other
-        column DDL feature that fills in a DB-generated identifier at INSERT-time
-        without requiring pre-execution of a SEQUENCE or other artifact.
+        column DDL feature that fills in a DB-generated identifier at
+        INSERT-time without requiring pre-execution of a SEQUENCE or other
+        artifact.
 
         """
         return skip_if(["firebird", "oracle", "postgresql", "sybase"],
-                "not supported by database"
-            )
+                       "not supported by database")
 
     @property
     def temporary_tables(self):
@@ -146,12 +163,11 @@ class DefaultRequirements(SuiteRequirements):
         PKs assuming they were reflected.
 
         this is essentially all the DBs in "identity" plus PostgreSQL, which
-        has SERIAL support.  FB and Oracle (and sybase?) require the Sequence to
-        be explicitly added, including if the table was reflected.
+        has SERIAL support.  FB and Oracle (and sybase?) require the Sequence
+        to be explicitly added, including if the table was reflected.
         """
         return skip_if(["firebird", "oracle", "sybase"],
-                "not supported by database"
-            )
+                       "not supported by database")
 
     @property
     def insert_from_select(self):
@@ -170,9 +186,7 @@ class DefaultRequirements(SuiteRequirements):
         """target database/driver can allow BLOB/BINARY fields to be compared
         against a bound parameter value.
         """
-        return skip_if(["oracle", "mssql"],
-                "not supported by database/driver"
-            )
+        return skip_if(["oracle", "mssql"], "not supported by database/driver")
 
     @property
     def binary_literals(self):
@@ -190,9 +204,7 @@ class DefaultRequirements(SuiteRequirements):
         # adding mssql here since it doesn't support comparisons either,
         # have observed generally bad behavior with binary / mssql.
 
-        return skip_if(["oracle", "mssql"],
-                "not supported by database/driver"
-            )
+        return skip_if(["oracle", "mssql"], "not supported by database/driver")
 
     @property
     def independent_cursors(self):
@@ -203,35 +215,34 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def independent_connections(self):
-        """Target must support simultaneous, independent database connections."""
+        """
+        Target must support simultaneous, independent database connections.
+        """
 
-        # This is also true of some configurations of UnixODBC and probably win32
-        # ODBC as well.
+        # This is also true of some configurations of UnixODBC and probably
+        # win32 ODBC as well.
         return skip_if([
-                    no_support("sqlite",
-                            "independent connections disabled "
-                                "when :memory: connections are used"),
-                    exclude("mssql", "<", (9, 0, 0),
-                            "SQL Server 2005+ is required for "
-                                "independent connections"
-                        )
-                    ]
-                )
+            no_support("sqlite",
+                       "independent connections disabled "
+                       "when :memory: connections are used"),
+            exclude("mssql", "<", (9, 0, 0),
+                    "SQL Server 2005+ is required for "
+                    "independent connections")])
 
     @property
     def updateable_autoincrement_pks(self):
         """Target must support UPDATE on autoincrement/integer primary key."""
 
         return skip_if(["mssql", "sybase"],
-                "IDENTITY columns can't be updated")
+                       "IDENTITY columns can't be updated")
 
     @property
     def isolation_level(self):
         return only_on(
-                    ('postgresql', 'sqlite', 'mysql', 'mssql'),
-                    "DBAPI has no isolation level support"
-                ) + fails_on('postgresql+pypostgresql',
-                          'pypostgresql bombs on multiple isolation level calls')
+            ('postgresql', 'sqlite', 'mysql', 'mssql'),
+            "DBAPI has no isolation level support") \
+            + fails_on('postgresql+pypostgresql',
+                       'pypostgresql bombs on multiple isolation level calls')
 
     @property
     def row_triggers(self):
@@ -244,7 +255,7 @@ class DefaultRequirements(SuiteRequirements):
 
             # huh?  TODO: implement triggers for PG tests, remove this
             no_support('postgresql',
-                    'PG triggers need to be implemented for tests'),
+                       'PG triggers need to be implemented for tests'),
         ])
 
     @property
@@ -253,15 +264,14 @@ class DefaultRequirements(SuiteRequirements):
         correlates to the parent."""
 
         return skip_if("oracle", 'Raises "ORA-01799: a column may not be '
-                    'outer-joined to a subquery"')
+                       'outer-joined to a subquery"')
 
     @property
     def update_from(self):
         """Target must support UPDATE..FROM syntax"""
 
         return only_on(['postgresql', 'mssql', 'mysql'],
-                "Backend does not support UPDATE..FROM")
-
+                       "Backend does not support UPDATE..FROM")
 
     @property
     def update_where_target_in_subquery(self):
@@ -276,8 +286,9 @@ class DefaultRequirements(SuiteRequirements):
                 FROM documents GROUP BY documents.user_id
             )
         """
-        return fails_if('mysql', 'MySQL error 1093 "Cant specify target table '
-                                        'for update in FROM clause"')
+        return fails_if('mysql',
+                        'MySQL error 1093 "Cant specify target table '
+                        'for update in FROM clause"')
 
     @property
     def savepoints(self):
@@ -319,6 +330,10 @@ class DefaultRequirements(SuiteRequirements):
                     "sqlite"
                 )
 
+    @property
+    def unique_constraint_reflection_no_index_overlap(self):
+        return self.unique_constraint_reflection + skip_if("mysql")
+
     @property
     def check_constraint_reflection(self):
         return fails_on_everything_except(
@@ -341,8 +356,7 @@ class DefaultRequirements(SuiteRequirements):
     def update_nowait(self):
         """Target database must support SELECT...FOR UPDATE NOWAIT"""
         return skip_if(["firebird", "mssql", "mysql", "sqlite", "sybase"],
-                "no FOR UPDATE NOWAIT support"
-            )
+                       "no FOR UPDATE NOWAIT support")
 
     @property
     def subqueries(self):
@@ -429,17 +443,18 @@ class DefaultRequirements(SuiteRequirements):
         return skip_if([
             no_support('firebird', 'no SA implementation'),
             no_support('mssql', 'two-phase xact not supported by drivers'),
-            no_support('oracle', 'two-phase xact not implemented in SQLA/oracle'),
+            no_support('oracle',
+                       'two-phase xact not implemented in SQLA/oracle'),
             no_support('drizzle', 'two-phase xact not supported by database'),
             no_support('sqlite', 'two-phase xact not supported by database'),
-            no_support('sybase', 'two-phase xact not supported by drivers/SQLA'),
+            no_support('sybase',
+                       'two-phase xact not supported by drivers/SQLA'),
             no_support('postgresql+zxjdbc',
-                    'FIXME: JDBC driver confuses the transaction state, may '
-                       'need separate XA implementation'),
+                       'FIXME: JDBC driver confuses the transaction state, '
+                       'may need separate XA implementation'),
             no_support('mysql',
-                'recent MySQL communiity editions have too many issues '
-                '(late 2016), disabling for now'),
-            ])
+                       'recent MySQL communiity editions have too many issues '
+                       '(late 2016), disabling for now')])
 
     @property
     def two_phase_recovery(self):
@@ -458,7 +473,9 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def empty_strings_varchar(self):
-        """target database can persist/return an empty string with a varchar."""
+        """
+        target database can persist/return an empty string with a varchar.
+        """
 
         return fails_if(["oracle"],
                         'oracle converts empty strings to a blank space')
@@ -479,7 +496,9 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def unicode_connections(self):
-        """Target driver must support some encoding of Unicode across the wire."""
+        """
+        Target driver must support some encoding of Unicode across the wire.
+        """
         # TODO: expand to exclude MySQLdb versions w/ broken unicode
         return skip_if([
             exclude('mysql', '<', (4, 1, 1), 'no unicode connection support'),
@@ -516,16 +535,16 @@ class DefaultRequirements(SuiteRequirements):
             "driver doesn't support 'sane' rowcount"
         )
 
-
-
     @property
     def emulated_lastrowid(self):
         """"target dialect retrieves cursor.lastrowid or an equivalent
         after an insert() construct executes.
         """
         return fails_on_everything_except('mysql',
-                                      'sqlite+pysqlite', 'sqlite+pysqlcipher',
-                                      'sybase', 'mssql')
+                                          'sqlite+pysqlite',
+                                          'sqlite+pysqlcipher',
+                                          'sybase',
+                                          'mssql')
 
     @property
     def implements_get_lastrowid(self):
@@ -540,8 +559,9 @@ class DefaultRequirements(SuiteRequirements):
 
         """
         return skip_if('mssql+pymssql', 'crashes on pymssql') + \
-                    fails_on_everything_except('mysql',
-                                       'sqlite+pysqlite', 'sqlite+pysqlcipher')
+            fails_on_everything_except('mysql',
+                                       'sqlite+pysqlite',
+                                       'sqlite+pysqlcipher')
 
     @property
     def sane_multi_rowcount(self):
@@ -560,7 +580,7 @@ class DefaultRequirements(SuiteRequirements):
         """Target driver reflects the name of primary key constraints."""
 
         return fails_on_everything_except('postgresql', 'oracle', 'mssql',
-                    'sybase', 'sqlite')
+                                          'sybase', 'sqlite')
 
     @property
     def array_type(self):
@@ -603,7 +623,7 @@ class DefaultRequirements(SuiteRequirements):
         datetime.datetime() with microsecond objects."""
 
         return skip_if(['mssql', 'mysql', 'firebird', '+zxjdbc',
-                    'oracle', 'sybase'])
+                        'oracle', 'sybase'])
 
     @property
     def datetime_historic(self):
@@ -646,8 +666,7 @@ class DefaultRequirements(SuiteRequirements):
         datetime.time() with microsecond objects."""
 
         return skip_if(['mssql', 'mysql', 'firebird', '+zxjdbc',
-                    'oracle', 'sybase'])
-
+                        'oracle', 'sybase'])
 
     @property
     def precision_numerics_general(self):
@@ -668,10 +687,9 @@ class DefaultRequirements(SuiteRequirements):
         to represent very large values."""
 
         return skip_if(
-                [("sybase+pyodbc", None, None,
-                    "Don't know how do get these values through FreeTDS + Sybase"),
-                ("firebird", None, None, "Precision must be from 1 to 18"),]
-            )
+            [("sybase+pyodbc", None, None,
+              "Don't know how do get these values through FreeTDS + Sybase"),
+             ("firebird", None, None, "Precision must be from 1 to 18")])
 
     @property
     def precision_numerics_many_significant_digits(self):
@@ -679,12 +697,18 @@ class DefaultRequirements(SuiteRequirements):
         such as 319438950232418390.273596, 87673.594069654243
 
         """
+        def cx_oracle_6_config(config):
+            return config.db.driver == "cx_oracle" and \
+                config.db.dialect.cx_oracle_ver >= (6, )
+
         return fails_if(
-                    [('sqlite', None, None, 'TODO'),
-                    ("firebird", None, None, "Precision must be from 1 to 18"),
-                    ("sybase+pysybase", None, None, "TODO"),
-                    ('mssql+pymssql', None, None, 'FIXME: improve pymssql dec handling')]
-                )
+            [cx_oracle_6_config,
+             ('sqlite', None, None, 'TODO'),
+             ("firebird", None, None, "Precision must be from 1 to 18"),
+             ("sybase+pysybase", None, None, "TODO"),
+             ('mssql+pymssql', None, None,
+              'FIXME: improve pymssql dec handling')]
+        )
 
     @property
     def precision_numerics_retains_significant_digits(self):
@@ -693,14 +717,14 @@ class DefaultRequirements(SuiteRequirements):
         the .000 maintained."""
 
         return fails_if(
-                [
-                    ('oracle', None, None,
-            "this may be a bug due to the difficulty in handling "
-            "oracle precision numerics"),
-                    ("firebird", None, None,
-        "database and/or driver truncates decimal places.")
-                ]
-                )
+            [
+                ('oracle', None, None,
+                 "this may be a bug due to the difficulty in handling "
+                 "oracle precision numerics"),
+                ("firebird", None, None,
+                 "database and/or driver truncates decimal places.")
+            ]
+        )
 
     @property
     def precision_generic_float_type(self):
@@ -708,35 +732,31 @@ class DefaultRequirements(SuiteRequirements):
         least seven decimal places when using the generic Float type."""
 
         return fails_if([
-                    ('mysql', None, None,
-                                'mysql FLOAT type only returns 4 decimals'),
-                    ('firebird', None, None,
-                                "firebird FLOAT type isn't high precision"),
-                ])
+            ('mysql', None, None,
+             'mysql FLOAT type only returns 4 decimals'),
+            ('firebird', None, None,
+             "firebird FLOAT type isn't high precision"),
+        ])
 
     @property
     def floats_to_four_decimals(self):
         return fails_if([
-                    ("mysql+oursql", None, None, "Floating point error"),
-                    ("firebird", None, None,
-                        "Firebird still has FP inaccuracy even "
-                        "with only four decimal places"),
-                    ('mssql+pyodbc', None, None,
-                                'mssql+pyodbc has FP inaccuracy even with '
-                                'only four decimal places '
-                            ),
-                    ('mssql+pymssql', None, None,
-                                'mssql+pymssql has FP inaccuracy even with '
-                                'only four decimal places '),
-                    (
-                        'postgresql+pg8000', None, None,
-                        'postgresql+pg8000 has FP inaccuracy even with '
-                        'only four decimal places '),
-                    (
-                        'postgresql+psycopg2cffi', None, None,
-                        'postgresql+psycopg2cffi has FP inaccuracy even with '
-                        'only four decimal places '),
-                ])
+            ("mysql+oursql", None, None, "Floating point error"),
+            ("firebird", None, None,
+             "Firebird still has FP inaccuracy even "
+             "with only four decimal places"),
+            ('mssql+pyodbc', None, None,
+             'mssql+pyodbc has FP inaccuracy even with '
+             'only four decimal places '),
+            ('mssql+pymssql', None, None,
+             'mssql+pymssql has FP inaccuracy even with '
+             'only four decimal places '),
+            ('postgresql+pg8000', None, None,
+             'postgresql+pg8000 has FP inaccuracy even with '
+             'only four decimal places '),
+            ('postgresql+psycopg2cffi', None, None,
+             'postgresql+psycopg2cffi has FP inaccuracy even with '
+             'only four decimal places ')])
 
     @property
     def fetch_null_from_numeric(self):
@@ -756,7 +776,7 @@ class DefaultRequirements(SuiteRequirements):
             try:
                 config.db.execute("SELECT 'a=>1,a=>2'::hstore;")
                 return True
-            except:
+            except Exception:
                 return False
 
         return only_if(check_hstore)
@@ -771,12 +791,11 @@ class DefaultRequirements(SuiteRequirements):
             try:
                 config.db.scalar("select '[1,2)'::int4range;")
                 return True
-            except:
+            except Exception:
                 return False
 
         return only_if(check_range_types)
 
-
     @property
     def oracle_test_dblink(self):
         return skip_if(
@@ -841,20 +860,19 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def order_by_label_with_expression(self):
         return fails_if([
-                    ('firebird', None, None, "kinterbasdb doesn't send full type information"),
-                    ('postgresql', None, None, 'only simple labels allowed'),
-                    ('sybase', None, None, 'only simple labels allowed'),
-                    ('mssql', None, None, 'only simple labels allowed')
-                ])
-
+            ('firebird', None, None,
+             "kinterbasdb doesn't send full type information"),
+            ('postgresql', None, None, 'only simple labels allowed'),
+            ('sybase', None, None, 'only simple labels allowed'),
+            ('mssql', None, None, 'only simple labels allowed')
+        ])
 
     @property
     def skip_mysql_on_windows(self):
         """Catchall for a large variety of MySQL on Windows failures"""
 
         return skip_if(self._has_mysql_on_windows,
-                "Not supported on MySQL + Windows"
-            )
+                       "Not supported on MySQL + Windows")
 
     @property
     def mssql_freetds(self):
@@ -868,6 +886,10 @@ class DefaultRequirements(SuiteRequirements):
             )
         )
 
+    @property
+    def ad_hoc_engines(self):
+        return exclusions.skip_if(["oracle"])
+
     @property
     def no_mssql_freetds(self):
         return self.mssql_freetds.not_()
@@ -875,7 +897,8 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def selectone(self):
         """target driver must support the literal statement 'select 1'"""
-        return skip_if(["oracle", "firebird"], "non-standard SELECT scalar syntax")
+        return skip_if(["oracle", "firebird"],
+                       "non-standard SELECT scalar syntax")
 
     @property
     def mysql_fsp(self):
@@ -888,16 +911,22 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def mysql_zero_date(self):
         def check(config):
-             row = config.db.execute("show variables like 'sql_mode'").first()
-             return not row or "NO_ZERO_DATE" not in row[1]
+            if not against(config, 'mysql'):
+                return False
+
+            row = config.db.execute("show variables like 'sql_mode'").first()
+            return not row or "NO_ZERO_DATE" not in row[1]
 
         return only_if(check)
 
     @property
     def mysql_non_strict(self):
         def check(config):
-             row = config.db.execute("show variables like 'sql_mode'").first()
-             return not row or "STRICT" not in row[1]
+            if not against(config, 'mysql'):
+                return False
+
+            row = config.db.execute("show variables like 'sql_mode'").first()
+            return not row or "STRICT_TRANS_TABLES" not in row[1]
 
         return only_if(check)
 
